package dareka;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.Arrays;
import java.util.Map;
import java.util.Properties;

import dareka.common.Config;
import dareka.common.Logger;
import dareka.processor.impl.Cache;

public class Main {
    // public so that external tools can read.
    public static final String VER_STRING = "NicoCache v0.22";

    // accessor for avoiding static link
    public static String getVersion() {
        return VER_STRING;
    }

    private static Server server;

    public static void stop() {
        if (server != null) {
            server.stop();
        }
    }

    public static void main(String[] args) {
        try {
            mainBody();
        } catch (Exception e) {
            Logger.debug(e);
        }

    }

    private static void mainBody() throws IOException {
        // [nl] iniɂlpBłmɂinit@CȂ
        File configFile = new File("config.ini");
        if (configFile.exists() == false) {
            configFile = new File("config.properties");
        }

        // ݒt@CȂăftHgݒt@CȂ烊l[Ďg
        if (configFile.exists() == false) {
            File defFile = new File("config.properties.default");
            if (defFile.exists()) {
                defFile.renameTo(configFile);
            }
        }

        Config config = configure(configFile);

        Logger.info(VER_STRING);
        Logger.info("port=" + Integer.getInteger("listenPort"));
        if (System.getProperty("proxyHost").equals("")) {
            Logger.info("direct mode (no secondary proxy)");
        } else {
            Logger.info("proxy host=" + System.getProperty("proxyHost"));
            Logger.info("proxy port=" + Integer.getInteger("proxyPort"));
        }
        Logger.info("title=" + Boolean.getBoolean("title"));

        if (Boolean.getBoolean("resumeDownload")) {
            Logger.info("Resume suspended download On");
        }

        if (Boolean.getBoolean(("touchCache"))) {
            Logger.info("Touch Cache File On");
        }

        if (Boolean.getBoolean("dareka.debug")) {
            Logger.info("debug mode");
        }

        Cache.init();
        // for resume
        // Cache.cleanup();
        Logger.info("total cache size=%,dbytes\n", Long.valueOf(Cache.size()));

        server = new Server(config);

        server.start();
    }

    private static Config configure(File configFile) throws IOException {
        Properties p = new Properties();
        setDefaults(p);

        Config config = new Config();
        if (configFile.exists()) {
            loadFrom(configFile, p);

            String v;

            v = p.getProperty(Config.LISTEN_PORT);
            if (v != null) {
                config.setListenPort(Integer.parseInt(v));
            }

            v = p.getProperty(Config.PROXY_HOST);
            if (v != null) {
                config.setProxyHost(v);
            }

            v = p.getProperty(Config.PROXY_PORT);
            if (v != null) {
                config.setProxyPort(Integer.parseInt(v));
            }

            v = p.getProperty(Config.TITLE);
            if (v != null) {
                config.setTitle(Boolean.parseBoolean(v));
            }
        } else {
            FileOutputStream out = new FileOutputStream(configFile);
            try { // ensure closing out
                p.store(out, "NicoCache config file");
            } finally {
                out.close();
            }
        }

        for (Map.Entry<Object, Object> entry : p.entrySet()) {
            String key = (String) entry.getKey();
            String value = (String) entry.getValue();
            System.setProperty(key, value);
        }

        return config;
    }

    private static void loadFrom(File configFile, Properties p)
            throws FileNotFoundException, IOException {
        FileInputStream in = new FileInputStream(configFile);
        try { // ensure closing in
            p.load(in);
        } finally {
            in.close();
        }
    }

    private static void setDefaults(Properties p) {
        try {
            setDefaultsFromFiles(p);
            if (p.size() > 0) {
                return;
            }
        } catch (IOException e) {
            Logger.debug(e);
        }

        p.setProperty("listenPort", "8080");
        p.setProperty("proxyHost", "");
        p.setProperty("proxyPort", "8081");
        p.setProperty("title", "true");
        p.setProperty("touchCache", "true");
    }

    private static void setDefaultsFromFiles(Properties p) throws IOException {
        File defaultsDir = new File("defaults");

        File[] files = defaultsDir.listFiles(new FilenameFilter() {
            public boolean accept(File dir, String name) {
                if (name.endsWith(".properties")) {
                    return true;
                } else {
                    return false;
                }
            }
        });

        if (files == null) {
            throw new IOException("faied to read: " + defaultsDir);
        }

        Arrays.sort(files);
        for (File f : files) {
            loadFrom(f, p);
        }
    }
}
