package dareka.processor;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.Socket;
import java.util.Map;

import dareka.common.Config;

public class StringResource extends Resource {
    private String statusline = "HTTP/1.1 200 OK";
    byte[] contentAsBytes;

    public StringResource(String resource) {
        contentAsBytes = getBytesAsUtf8(resource);
    }

    public StringResource(byte[] resource) {
        contentAsBytes = resource;
    }

    public StringResource(int status, String resource) {
        switch (status) {
        case 302:
            statusline = "HTTP/1.1 302 Found\r\nLocation:" + resource;
            contentAsBytes = new byte[0]; // represents it's not forgotten.
            return;
        case 304:
            statusline = "HTTP/1.1 304 Not Modified";
            break;
        case 404:
            statusline = "HTTP/1.1 404 Not Found";
            break;
        }
        contentAsBytes = getBytesAsUtf8(resource);
    }

    private byte[] getBytesAsUtf8(String str) {
        try {
            return str.getBytes("UTF-8");
        } catch (UnsupportedEncodingException e) {
            // never happen
            return null;
        }
    }

    public void addNoCacheResponseHeaders() {
        setResponseHeader("Expires", "Thu, 01 Dec 1994 16:00:00 GMT");
        setResponseHeader("Cache-Control", "no-store");
        setResponseHeader("Pragma", "no-cache");
    }

    public void copyResponseHeaders(HttpResponseHeader responseHeader) {
        for (Map.Entry<String, String> entry : responseHeader.getMessageHeaders().entrySet()) {
            setResponseHeader(entry.getKey(), entry.getValue());
        }
    }

    @Override
    public boolean transferTo(Socket receiver, HttpRequestHeader requestHeader,
            Config config) throws IOException {
        HttpResponseHeader responseHeader = getResponseHeader();

        overrideResponseMessage(responseHeader);

        fireOnResonseHeader(responseHeader);

        HttpUtil.sendHeader(receiver, responseHeader);

        HttpUtil.sendBody(receiver.getOutputStream(), new ByteArrayInputStream(
                contentAsBytes), contentAsBytes.length);

        return true;
    }

    // default access for test. 
    HttpResponseHeader getResponseHeader() throws IOException {
        HttpResponseHeader responseHeader =
                new HttpResponseHeader(statusline + "\r\n\r\n");
        responseHeader.setMessageHeader(HttpHeader.CONNECTION, "keep-alive");
        responseHeader.setContentLength(contentAsBytes.length);
        return responseHeader;
    }

}
