package dareka.processor.impl;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.concurrent.Executor;
import java.util.concurrent.FutureTask;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import dareka.common.Logger;
import dareka.processor.HttpHeader;
import dareka.processor.HttpRequestHeader;
import dareka.processor.Processor;
import dareka.processor.Resource;

public class NicoCachingProcessor implements Processor {
    private static final String[] SUPPORTED_METHODS = new String[] { "GET" };
    /**
     * SMILEVIDEO̓URL̐K\
     */
    // http://s-clb5.smilevideo.jp/smile?v=102982.92382
    private static final Pattern SM_FLV_PATTERN =
            Pattern.compile("^http://[^/]+(?:smilevideo|nicovideo)\\.jp/smile\\?v=([^.]+)\\.\\d+(low)?$");

    private Executor executor;

    public NicoCachingProcessor(Executor executor) {
        this.executor = executor;
    }

    public String[] getSupportedMethods() {
        return SUPPORTED_METHODS;
    }

    public Pattern getSupportedURLAsPattern() {
        return SM_FLV_PATTERN;
    }

    public String getSupportedURLAsString() {
        return null;
    }

    public Resource onRequest(HttpRequestHeader requestHeader)
            throws IOException {
        Matcher m = SM_FLV_PATTERN.matcher(requestHeader.getURI());
        if (!m.find()) {
            // it must not happen...
            return Resource.get(Resource.Type.URL, requestHeader.getURI());
        }

        String smId = "sm" + m.group(1);
        String suffix = m.group(2);

        Cache cache = new Cache(smId);
        if (!cache.exists() && suffix != null) {
            cache = new Cache(smId + suffix);
        }

        if (cache.exists()) {
            Logger.info("using cache: " + cache.getCacheFileName());
            if (Boolean.getBoolean("touchCache"))
                cache.touch();
            Resource r = Resource.get(Resource.Type.URL, cache.getURLString());

            r.setResponseHeader(HttpHeader.CONTENT_TYPE, "video/flv");

            return r;
        }

        FutureTask<String> retrieveTitlteTask = null;
        if (Boolean.getBoolean("title")) {
            retrieveTitlteTask =
                    new FutureTask<String>(new NicoCachingTitleRetriever(smId));
            executor.execute(retrieveTitlteTask);
        }

        Logger.info("no cache found: " + cache.getCacheFileName());

        // [nl] uEŨW[ƁANCł̃W[Ή
        requestHeader.removeMessageHeader("Range");
        requestHeader.removeMessageHeader("If-Range");
        long tmpSize = cache.tmpLength();
        InputStream cacheInput = null;
        // [nl] dDLɂȂꍇ́A2߂̃RlNVł͎Ă镪̃LbV𗘗p
        //		܂A2߂̃RlNV͗DLȂBAIɃW[
        //		TODO:r邩ȂA܂͂
        boolean dupconnect = false;
        if (Cache.getDLFlag(cache.getId()))
            dupconnect = true;
        // [nl] W[[
        if (tmpSize != 0
                && (dupconnect || Boolean.getBoolean("resumeDownload"))) {
            tmpSize = cache.tmpLength();
            requestHeader.setMessageHeader("Range", "bytes="
                    + String.valueOf(tmpSize) + "-");
            cacheInput = new BufferedInputStream(cache.getTmpInputStream());
        }
        // [nl] DLXgɓ
        Cache.setDLFlag(cache.getId(), Integer.MAX_VALUE);

        Resource r = Resource.get(Resource.Type.URL, requestHeader.getURI());
        r.addTransferListener(new NicoCachingListener(cache,
                retrieveTitlteTask, cacheInput, dupconnect));

        return r;
    }
}
